import planarity
# Example of the complete graph of 5 nodes, K5
# K5 is not planar
# any of the following formats can bed used for representing the graph

edgelist = [(0, 1), (0, 2), (0, 3), (0, 4),
            (1, 2),(1, 3),(1, 4),
            (2, 3), (2, 4),
            (3, 4)]

dictofdicts = {0: {1: {}, 2: {}, 3: {}, 4: {}},
               1: {2: {}, 3: {}, 4: {}},
               2: {3: {}, 4: {}},
               3: {4: {}},
               4: {}}

dictofsets = {0: set([1,2,3,4]),
              1: set([2,3,4]),
              2: set([3,4]),
              3: set([4]),
              4: set([])}

dictoflists = {0: list([1,2,3,4]),
               1: list([2,3,4]),
               2: list([3,4]),
               3: list([4]),
               4: list([])}

print planarity.is_planar(edgelist)  # False
print planarity.is_planar(dictofdicts) # False
print planarity.is_planar(dictofsets) # False
print planarity.is_planar(dictoflists) # False



"""Interface for Boyer's (C) planarity algorithms."""
cdef extern from "src/graphStructures.h":
    ctypedef struct baseGraphStructure:
        pass
    ctypedef baseGraphStructure * graphP

    ctypedef struct edgeRec:
        pass
    ctypedef edgeRec * edgeRecP

    cdef int gp_GetFirstVertex(graphP theGraph)
    cdef int gp_GetLastVertex(graphP theGraph) 
    cdef int gp_GetFirstArc(graphP theGraph, int v)
    cdef int gp_GetLastArc(graphP theGraph, int v)
    cdef int gp_IsArc(int v) 
    cdef int gp_GetNeighbor(graphP theGraph, int v) 
    cdef int gp_GetPrevArc(graphP theGraph, int v)
    cdef int gp_GetNextArc(graphP theGraph, int v)
    cdef int gp_GetDirection(graphP theGraph, int v)

cdef extern from "src/graph.h":
    cdef int OK, NOTOK, NULL 
    cdef int EMBEDFLAGS_PLANAR, NONEMBEDDABLE, EMBEDFLAGS_DRAWPLANAR
    cdef int WRITE_ADJLIST
    cdef int EDGEFLAG_DIRECTION_INONLY, EDGEFLAG_DIRECTION_OUTONLY  

    cdef graphP gp_New()
    cdef void gp_Free(graphP *pGraph)
    cdef int gp_InitGraph(graphP theGraph, int N)
    cdef int gp_AddEdge(graphP theGraph, int u, int ulink, int v, int vlink)
    cdef int gp_Embed(graphP theGraph, int embedFlags)
    cdef int gp_Write(graphP theGraph, char *FileName, int Mode)
    cdef void gp_SortVertices(graphP theGraph)


cdef extern from "src/graphDrawPlanar.h":
    cdef int gp_DrawPlanar_RenderToString(graphP theEmbedding, char **pRenditionString);
    cdef int gp_AttachDrawPlanar(graphP theGraph)


cdef extern from "src/graphDrawPlanar.private.h":
    ctypedef struct DrawPlanar_VertexInfo:
       int pos
       int start
       int end
    ctypedef DrawPlanar_VertexInfo * DrawPlanar_VertexInfoP

    ctypedef struct DrawPlanar_EdgeRec:
       int pos
       int start
       int end
    ctypedef DrawPlanar_EdgeRec * DrawPlanar_EdgeRecP

    ctypedef struct DrawPlanarContext:
        DrawPlanar_EdgeRecP E
        DrawPlanar_VertexInfoP VI

cdef extern from "src/graphExtensions.h":
    cdef void * gp_GetExtension(graphP theGraph, int moduleID)
    cdef int gp_FindExtension(graphP theGraph, int moduleID, void *pContext)



"""NetworkX interface to planarity."""
import planarity

__all__ = ['kuratowski_subgraph', 'pgraph_graph',
           'networkx_graph', 'draw']

def kuratowski_subgraph(graph):
    """Return forbidden subgraph of nonplanar graph G."""
    try:
        import networkx as nx
    except ImportError:
        raise ImportError("NetworkX required for kuratowski_subgraph()")
    pgraph = planarity.PGraph(graph)
    edges = pgraph.kuratowski_edges()
    return nx.Graph(edges)

def networkx_graph(pgraph):
    """Return NetworkX graph built from planarity pgraph."""
    try:
        import networkx as nx
    except ImportError:
        raise ImportError("NetworkX required for networkx_graph()")
    graph = nx.Graph()
    graph.add_nodes_from(pgraph.nodes(data=True))
    graph.add_edges_from(pgraph.edges(data=True))
    return graph

def pgraph_graph(graph):
    """Return pgraph graph built from NetworkX graph."""
    return planarity.PGraph(graph)

def draw(graph, labels=True):
    """Draw planar graph with Matplotlib."""
    try:
        import matplotlib.pyplot as plt
        from matplotlib.patches import Circle
        from matplotlib.collections import PatchCollection
    except ImportError:
        raise ImportError("Matplotlib is required for draw()")
    pgraph = planarity.PGraph(graph)
    pgraph.embed_drawplanar()
    hgraph = networkx_graph(pgraph)
    patches = []
    node_labels = {}
    xs = []
    ys = []
    for node, data in hgraph.nodes(data=True):
        y = data['pos']
        xb = data['start']
        xe = data['end']
        x = int((xe+xb)/2)
        node_labels[node] = (x, y)
        patches += [Circle((x, y), 0.25)]#,0.5,fc='w')]
        xs.extend([xb, xe])
        ys.append(y)
        plt.hlines([y], [xb], [xe])

    for (_, _, data) in hgraph.edges(data=True):
        x = data['pos']
        yb = data['start']
        ye = data['end']
        ys.extend([yb, ye])
        xs.append(x)
        plt.vlines([x], [yb], [ye])

    # labels
    if labels:
        for n, (x, y) in node_labels.items():
            plt.text(x, y, n,
                     horizontalalignment='center',
                     verticalalignment='center',
                     bbox = dict(boxstyle='round',
                                 ec=(0.0, 0.0, 0.0),
                                 fc=(1.0, 1.0, 1.0),
                                 )
                     )
    p = PatchCollection(patches)
    ax = plt.gca()
    ax.add_collection(p)
    plt.axis('equal')
    plt.xlim(min(xs)-1, max(xs)+1)
    plt.ylim(min(ys)-1, max(ys)+1)



import os
import sys

import setuptools
from setuptools import setup

from distutils.extension import Extension
from glob import glob

try:
    from Cython.Build import cythonize
except ImportError:
    USE_CYTHON = False
else:
    USE_CYTHON = True

long_description  = """Graph planarity tools.
A wrapper to Boyer's (C) planarity algorithms: http://code.google.com/p/planarity/

Provides planarity testing, forbidden subgraph finding, and planar embeddings.

Works with the NetworkX graph package (not required).
"""

classifiers = [
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Scientific/Engineering :: Information Analysis',
        'Topic :: Scientific/Engineering :: Mathematics',
        'Topic :: Scientific/Engineering :: Physics']

ext = '.pyx' if USE_CYTHON else '.c'

sourcefiles = ['planarity/planarity'+ext]
sourcefiles.extend(glob("planarity/src/*.c"))

extensions = [Extension("planarity.planarity",
                        sourcefiles,
                        include_dirs=['planarity/src/'],
                        )]

if USE_CYTHON:
    from Cython.Build import cythonize
    extensions = cythonize(extensions)

setup(
    name= 'planarity',
    packages=setuptools.find_packages(),
    maintainer = 'Aric Hagberg',
    maintainer_email = 'aric.hagberg@gmail.com',
    author = 'Aric Hagberg',
    author_email = 'aric.hagberg@gmail.com',
    description = 'Graph planarity tools.',
    classifiers = classifiers,
    long_description = long_description,
    license = 'BSD',
    ext_modules = extensions,
    version          = '0.5',
    url = 'https://github.com/hagberg/planarity/',
    download_url='https://pypi.python.org/pypi/planarity',
    package_data = {'planarity':['tests/*.py']},
    zip_safe = False
    )




import planarity
import networkx as nx
# Example of the complete graph of 5 nodes, K5
G=nx.complete_graph(5)
# K5 is not planar
print(planarity.is_planar(G)) # False
# find forbidden Kuratowski subgraph
K=planarity.kuratowski_subgraph(G)
print(K.edges()) # K5 edges

